"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
process.env.NODE_ENV = "production";
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const glob_1 = require("glob");
const utils_1 = require("./utils");
const vinyl_fs_1 = tslib_1.__importDefault(require("vinyl-fs"));
const prettier_1 = tslib_1.__importDefault(require("prettier"));
const fs_1 = require("fs");
class CompileDemoProject {
    constructor(param) {
        var _a;
        this.isIgnoreFolderName = (folderName) => {
            return this.ignoreDemoProjectFolders.includes(folderName);
        };
        this.removeTemporaryFiles = () => {
            fs_extra_1.removeSync(this.megaScriptConfigPath);
            fs_extra_1.removeSync(this.templateToCopyPath);
        };
        this.copyTemplateProject = () => {
            fs_extra_1.ensureDirSync(this.templateToCopyPath);
            return new Promise((resolve, reject) => {
                vinyl_fs_1.default
                    .src([
                    `node_modules/${this.templateName}/**/*`,
                    `!node_modules/${this.templateName}/node_modules/**/*`
                ], { cwd: this.projectPath })
                    .pipe(vinyl_fs_1.default.dest(this.templateToCopyPath))
                    .on("end", () => {
                    resolve(true);
                })
                    .on("error", data => {
                    reject(data);
                });
            });
        };
        this.getAPITableData = (folderName, childrenWidget) => {
            let importInfo = "";
            let demo = "";
            const importModuleName = utils_1.fixFolderName(folderName);
            importInfo = `import ${importModuleName} from '../../../../src/widgets/${folderName}/lugia.${folderName}.zh-CN.json';`;
            demo = `<EditTables_Package dataSource={${importModuleName}} />`;
            if (childrenWidget) {
                childrenWidget.forEach(item => {
                    const childrenMoudleName = utils_1.fixFolderName(item);
                    importInfo = `${importInfo} import ${childrenMoudleName} from '../../../../src/widgets/${folderName}/lugia.${item}.zh-CN.json';`;
                    demo = `${demo}<EditTables_Package dataSource={${childrenMoudleName}} />`;
                });
            }
            return { importInfo, demo };
        };
        this.calculateFileExtension = (path) => {
            const extensionTypes = ["js", "jsx", "tsx"];
            const fullFilePaths = extensionTypes.map(item => `${path}.${item}`);
            let fullFilePath = "";
            fullFilePaths.some(filePath => {
                const fileExists = fs_extra_1.existsSync(filePath);
                if (fileExists) {
                    fullFilePath = filePath;
                }
                return fileExists;
            });
            return fullFilePath;
        };
        this.getImportInfoAndDemo = (demoNames, config, folderName) => {
            let importInfo = "";
            let demo = "";
            let link = "";
            let index = 0;
            for (const item of demoNames) {
                const { title: titleStr, desc: descStr } = config[item];
                const withoutExtensionFilePath = path_1.join(this.sourceWidgetsPath, folderName, "demo", item);
                const fileExtension = this.calculateFileExtension(withoutExtensionFilePath);
                if (fileExtension) {
                    const codeStr = fs_extra_1.readFileSync(fileExtension).toString("utf-8");
                    importInfo = `${importInfo} const ${item} =  require('../../../../src/widgets/${folderName}/demo/${item}').default;`;
                    const titleTxt = utils_1.toText(titleStr);
                    const descTxt = utils_1.toText(descStr);
                    demo = `${demo}<Demo_Package title={'${titleTxt}'} titleID={'${folderName}-${index}'} code={<code>{ \`${utils_1.toText(codeStr)}\`}</code>} desc={'${descTxt}'}  demo={<${item} />}></Demo_Package>`;
                    link = `${link}<Link_Package title={'${titleTxt}'} href={'#${folderName}-${index}'} />`;
                    index++;
                }
            }
            return { importInfo, demo, link };
        };
        this.getThemeConfig = (folderName, childrenWidget) => {
            const fixeMoudleName = utils_1.fixFolderName(folderName);
            const themeData = [];
            let demo = "";
            themeData.push(fixeMoudleName);
            if (childrenWidget) {
                childrenWidget.forEach(item => {
                    const childrenMoudleName = utils_1.fixFolderName(item);
                    themeData.push(childrenMoudleName);
                });
            }
            demo = `<EditTheme_Package dataSource={{${themeData.join(",")}}}/>`;
            return { demo };
        };
        this.getDemoPageContent = (param) => {
            const { demoNames, config, folderName, pageInfo, childrenWidget } = param;
            const { title: pageTitle, subTitle, desc: pageDesc } = pageInfo;
            const { importInfo: ApiImport, demo: ApiTable } = this.getAPITableData(folderName, childrenWidget);
            const importInfoAndDemo = this.getImportInfoAndDemo(demoNames, config, folderName);
            const { demo: ThemeTable } = this.getThemeConfig(folderName, childrenWidget);
            return `/* eslint-disable */
      import  React from 'react';
     import Grid_Package from '@lugia/lugia-web/dist/grid';
     import Anchor_Package from '@lugia/lugia-web/dist/anchor';
     import EditTables_Package from '../../edit-table';
     import EditTheme_Package from '../../edit-theme';
     import FooterNav_Package from '../../footer-nav';
     import PageNavHoC_Package from '../../common/PageNavHoC';
     import widgetrouter_Package from '../../router/widgetrouter';
      ${ApiImport}
      import Demo_Package from '../code-box';
      import Title_Package from '../code-box/Title';
      ${importInfoAndDemo.importInfo}
      const { Link_Package } = Anchor_Package;
      const { Row_Package, Col_Package } = Grid_Package;
      export default PageNavHoC_Package(widgetrouter_Package, class ComDemo extends React.Component {
          handleLinkClick = (e, href) => {
            if (href) {
              const name = href.slice(1);
              const anchorElement = document.getElementById(name);
              if (anchorElement) {
                anchorElement.scrollIntoView({ block: 'start', behavior: 'smooth' });
              }
            }
          };
          render() {
              const {next, prev, isMobile = false} = this.props;
              const span = isMobile ? 24 : 20;
              const style = isMobile ? {} : {paddingRight: '50px'};
              return (
                  <Row_Package>
                      <Col_Package span={span}>
                        <div style={style}>
                            <Title_Package title={'${pageTitle}'} subTitle={'${subTitle}'} desc={'${pageDesc}'} />
                            ${importInfoAndDemo.demo}
                            ${ApiTable}
                            ${ThemeTable}
                            <FooterNav_Package prev={prev} next={next} />
                          </div>
                      </Col_Package>
                      {!isMobile && <Col_Package span={4}>
                          <Anchor_Package  slideType="line" onClick={this.handleLinkClick} useHref={false}>
                              ${importInfoAndDemo.link}
                          </Anchor_Package>
                      </Col_Package>}
                  </Row_Package>
              )
          }
       });
      `;
        };
        this.generateEntryFile = (meta, config, folderName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { childrenWidget = [] } = meta;
            const subTitle = meta.widgetName;
            let demoPageContent = "";
            try {
                const demoNames = Object.keys(config);
                demoPageContent = this.getDemoPageContent({
                    demoNames,
                    config,
                    folderName,
                    pageInfo: {
                        title: meta.title,
                        subTitle,
                        desc: meta.desc
                    },
                    childrenWidget
                });
            }
            catch (error) {
                console.log("(%s) 代码生成错误 ", folderName, error.message);
                return;
            }
            try {
                const path = path_1.join(this.templateProjectWidgetPath, folderName, "index.js");
                fs_extra_1.ensureFileSync(path);
                fs_1.writeFileSync(path, prettier_1.default.format(demoPageContent, {
                    useTabs: false,
                    printWidth: 80,
                    tabWidth: 2,
                    singleQuote: true,
                    trailingComma: "none",
                    bracketSpacing: true,
                    jsxBracketSameLine: false,
                    semi: true,
                    parser: "babel"
                }));
            }
            catch (err) {
                console.log("(%s) 写入文件失败", folderName, err);
            }
        });
        this.generateSingleDemoPage = (folderName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const metaUrl = path_1.join(this.sourceWidgetsPath, folderName, `lugia.${folderName}.zh-CN.json`);
            const configUrl = path_1.join(this.sourceWidgetsPath, folderName, "demo", "config.json");
            const meta = utils_1.requireLoadData(metaUrl);
            const config = utils_1.requireLoadData(configUrl);
            if (meta && config) {
                yield this.generateEntryFile(meta, config, folderName);
                return;
            }
            console.log("(%s) 生成demoPage失败,缺少meta信息或demo中缺少config配置信息", folderName);
        });
        this.getMenuRouter = (data) => {
            const res = [];
            res.push("{");
            data.forEach(item => {
                const { children } = item;
                children &&
                    children.forEach(categoryItem => {
                        const { value, text, floderName, sort } = categoryItem;
                        res.push(`
      '${value}':
        {
          value: '${value}',
          sort: ${sort},
          text: '${text}',
          exact: true,
          render: async () => {
            return import('../widgets/${floderName}');
          },
        },`);
                    });
            });
            res.push(" }");
            return res.join("");
        };
        this.generateDemoRouter = (folderNames) => {
            const router = this.getRouterData(folderNames, item => `${item.widgetName} ${item.title}`);
            fs_1.writeFileSync(path_1.join(this.templateProjectRouterPath, "widgets.js"), ` /* eslint-disable */ \n export default ${JSON.stringify(router)}`);
            fs_1.writeFileSync(path_1.join(this.templateProjectRouterPath, "widgetrouter.js"), ` /* eslint-disable */ export default ${this.getMenuRouter(router)}`);
        };
        this.getRouterData = (folderNames, getText) => {
            const category2WidgetNames = {};
            try {
                folderNames.forEach(folderName => {
                    const metaUrl = path_1.join(this.sourceWidgetsPath, folderName, `lugia.${folderName}.zh-CN.json`);
                    const meta = utils_1.requireLoadData(metaUrl);
                    if (!meta) {
                        return;
                    }
                    const { category = [] } = meta;
                    category.forEach((item) => {
                        var _a;
                        const categoryData = (_a = category2WidgetNames[item]) !== null && _a !== void 0 ? _a : [];
                        if (categoryData.length === 0) {
                            category2WidgetNames[item] = categoryData;
                        }
                        meta.floderName = folderName;
                        categoryData.push(meta);
                    });
                });
                let index = 35;
                return Object.keys(category2WidgetNames).map(key => {
                    const widgets = category2WidgetNames[key];
                    const children = widgets &&
                        widgets.map(meta => {
                            const { widgetName, floderName } = meta;
                            return {
                                widgetName,
                                sort: index++,
                                floderName,
                                value: utils_1.getRouterUrl(widgetName),
                                text: getText(meta)
                            };
                        });
                    return {
                        text: key,
                        category: key,
                        describe: true,
                        children
                    };
                });
            }
            catch (error) {
                console.log("%s 异常  X", error);
                return [];
            }
        };
        this.generateSearchIndex = (folderNames) => {
            const res = [];
            const widget = "组件";
            const router = this.getRouterData(folderNames, item => item.title);
            router.forEach(categoryData => {
                const { children } = categoryData;
                if (children) {
                    children.forEach(item => {
                        const { value: url, text, widgetName } = item;
                        if (url) {
                            text &&
                                res.push({
                                    url,
                                    type: widget,
                                    owner: widgetName,
                                    content: text,
                                    power: 2
                                });
                            widgetName &&
                                res.push({
                                    url,
                                    type: widget,
                                    owner: widgetName,
                                    content: widgetName,
                                    power: 1
                                });
                        }
                    });
                }
            });
            folderNames.forEach(folderName => {
                const metaUrl = path_1.join(this.sourceWidgetsPath, folderName, `lugia.${folderName}.zh-CN.json`);
                const configUrl = path_1.join(this.sourceWidgetsPath, folderName, "demo", "config.json");
                const meta = utils_1.requireLoadData(metaUrl);
                const config = utils_1.requireLoadData(configUrl);
                if (meta && config) {
                    const { widgetName } = meta;
                    const url = utils_1.getRouterUrl(widgetName);
                    Object.keys(config || {}).forEach((demoName, i) => {
                        const configValue = config[demoName];
                        const { title, desc } = configValue;
                        const link = `${url}#${folderName}-${i}`;
                        title &&
                            res.push({
                                owner: widgetName,
                                url: link,
                                type: widget,
                                content: title,
                                power: 3
                            });
                        desc &&
                            res.push({
                                owner: widgetName,
                                url: link,
                                type: widget,
                                content: desc,
                                power: 4
                            });
                    });
                }
            });
            fs_1.writeFileSync(path_1.join(this.templateProjectRouterPath, "search.js"), `/* eslint-disable */  module.exports = ${JSON.stringify(res)}`);
        };
        this.generateMegaScriptConfigFile = () => {
            const aliasConfigs = this.alias &&
                Object.keys(this.alias).map((aliasName) => {
                    const isRightAlias = aliasName && this.alias[aliasName];
                    return isRightAlias
                        ? `'${aliasName}': join(__dirname, '${this.alias[aliasName]}')`
                        : "";
                });
            const aliasStr = aliasConfigs.length > 0
                ? ["alias:", "{", ...aliasConfigs, "}", ","].join("")
                : "";
            const publicPath = this.publicPath
                ? `publicPath: '${this.publicPath}',`
                : "";
            const configContent = `const {join} =  require('path'); \n
    exports.default = {
      ${publicPath}
      entry: '${this.entryFile}',
      outputPath: '${this.demoProjectOutputPath}',
      disableCSSModules: true,
      cssModulesWithAffix: true,
      extraBabelIncludes:[${this.extraBabelIncludes}],
      extraBabelPlugins:${JSON.stringify(this.extraBabelPlugins)},
      ${aliasStr}
    };`;
            fs_extra_1.ensureFileSync(this.megaScriptConfigPath);
            fs_1.writeFileSync(this.megaScriptConfigPath, configContent);
        };
        this.generateDemoPage = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                glob_1.glob(`${this.sourceWidgetsPath}/*/demo/config.*(json)`, { cwd: this.sourceWidgetsPath }, (err, files) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    if (err) {
                        console.info("错误信息：", err);
                        reject(false);
                        return;
                    }
                    try {
                        const matchFolderName = files.map(file => path_1.parse(path_1.join(file, "../../")).base);
                        const filterFolderName = matchFolderName.filter(folderName => !this.isIgnoreFolderName(folderName));
                        for (const folderName of filterFolderName) {
                            yield this.generateSingleDemoPage(folderName);
                        }
                        this.generateDemoRouter(filterFolderName);
                        this.generateSearchIndex(filterFolderName);
                        resolve(true);
                    }
                    catch (e) {
                        console.log("错误信息：", e);
                        resolve(false);
                    }
                }));
            });
        });
        this.compile = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                fs_extra_1.removeSync(this.templateToCopyPath);
                const packageInfo = {
                    projectPath: this.projectPath,
                    packageInfo: {
                        name: this.templateName
                    },
                    registry: this.npmRegistry
                };
                yield utils_1.installSinglePackage(packageInfo);
                yield this.copyTemplateProject();
                yield utils_1.unInstallSinglePackage(packageInfo);
                yield this.generateDemoPage();
                yield utils_1.installNpmPackage(this.npmRegistry, this.templateToCopyPath);
                yield this.generateMegaScriptConfigFile();
                const { code, message } = yield utils_1.buildDemoProject(this.cwd, this.megaScriptConfigPath);
                if (!code) {
                    return { code: 0, message };
                }
                this.isRemoveTemporaryFiles && this.removeTemporaryFiles();
                return { code: 1 };
            }
            catch (e) {
                return { code: 0, message: e.message };
            }
        });
        const { cwd, npmRegistry, templateName = "@lugia/lugia-widget-demo", templateToCopyPath, projectPath, demoProjectOutputPath = "dist", megaScriptConfigPath, ignoreDemoProjectFolders = ["code-box"], isRemoveTemporaryFiles = true, sourceWidgetsPath = path_1.join(projectPath, "src", "widgets"), entryFile = "createDemo/src/index.js", extraBabelIncludes = [], alias = {}, extraBabelPlugins = [], publicPath } = param;
        this.projectPath = projectPath;
        this.npmRegistry = npmRegistry;
        this.templateName = templateName;
        this.templateToCopyPath = templateToCopyPath;
        this.demoProjectOutputPath = demoProjectOutputPath;
        this.sourceWidgetsPath = sourceWidgetsPath;
        this.templateProjectSrcPath = path_1.join(this.templateToCopyPath, "src");
        this.templateProjectWidgetPath = path_1.join(this.templateProjectSrcPath, "widgets");
        this.templateProjectRouterPath = path_1.join(this.templateProjectSrcPath, "router");
        this.megaScriptConfigPath = megaScriptConfigPath !== null && megaScriptConfigPath !== void 0 ? megaScriptConfigPath : path_1.join(this.templateToCopyPath, "lugia.config.js");
        this.ignoreDemoProjectFolders = ignoreDemoProjectFolders;
        this.isRemoveTemporaryFiles = isRemoveTemporaryFiles;
        this.cwd = (_a = cwd !== null && cwd !== void 0 ? cwd : projectPath) !== null && _a !== void 0 ? _a : process.cwd();
        this.entryFile = entryFile;
        this.extraBabelIncludes = extraBabelIncludes;
        this.extraBabelPlugins = extraBabelPlugins;
        this.alias = alias;
        this.publicPath = publicPath;
    }
}
exports.default = CompileDemoProject;
process.on("uncaughtException", err => {
    console.log("uncaughtException", err);
});
