"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.requireLoadData = exports.getRouterUrl = exports.toText = exports.fixFolderName = exports.buildDemoProject = exports.installNpmPackage = exports.installSinglePackage = exports.unInstallSinglePackage = void 0;
const tslib_1 = require("tslib");
const execa_1 = tslib_1.__importDefault(require("execa"));
const buildApp_1 = tslib_1.__importDefault(require("@lugia/mega-scripts/lib/utils/buildApp"));
function execaActuator(path, command, commandParam) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return execa_1.default(command, commandParam, {
            cwd: path,
            execPath: path,
            env: Object.assign(Object.assign({}, process.env), { NODE_ENV: "dev" })
        });
    });
}
function unInstallSinglePackage(param) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { projectPath, packageInfo } = param;
        const { name } = packageInfo;
        yield execaActuator(projectPath, "yarn", ["remove", `${name}`]);
    });
}
exports.unInstallSinglePackage = unInstallSinglePackage;
function installSinglePackage(param) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { projectPath, packageInfo, registry = "https://registry.npmjs.org/" } = param;
        const { name, version = "latest" } = packageInfo;
        yield execaActuator(projectPath, "yarn", [
            "add",
            `${name}@${version}`,
            "--registry",
            registry
        ]);
    });
}
exports.installSinglePackage = installSinglePackage;
function installNpmPackage(registry, projectPath) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield execaActuator(projectPath, "yarn", ["--registry", registry]);
    });
}
exports.installNpmPackage = installNpmPackage;
function buildDemoProject(projectPath, megaScriptConfigPath) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            buildApp_1.default({
                cwd: projectPath,
                watch: false,
                configFile: megaScriptConfigPath
            }, (err) => {
                if (err) {
                    reject({ code: 0, message: err.message });
                }
                resolve({ code: 1 });
            });
        });
    });
}
exports.buildDemoProject = buildDemoProject;
function fixFolderName(folderName) {
    return folderName.replace(/-/g, "").toUpperCase();
}
exports.fixFolderName = fixFolderName;
function toText(str) {
    return str
        .replace(/\$/g, String.raw `\$`)
        .replace(/\`/g, String.raw `\``)
        .replace(/\'/g, String.raw `\'`)
        .replace(/\"/g, String.raw `\"`)
        .replace(/\n/g, String.raw `\n`);
}
exports.toText = toText;
function getRouterUrl(widgetName) {
    return `/component/${widgetName.toLowerCase()}`;
}
exports.getRouterUrl = getRouterUrl;
function requireLoadData(url) {
    try {
        return require(url);
    }
    catch (e) {
        return undefined;
    }
}
exports.requireLoadData = requireLoadData;
